@@ -18,7 +18,7 @@
    along with this program.  If not, see <http://www.gnu.org/licenses/>.  */
 
 /* NVIDIA CUDA Debugger CUDA-GDB
-   Copyright (C) 2007-2024 NVIDIA Corporation
+   Copyright (C) 2007-2025 NVIDIA Corporation
    Modified from the original GDB file referenced above by the CUDA-GDB
    team at NVIDIA <cudatools@nvidia.com>. */
 
@@ -169,7 +169,7 @@ static const option_def gcore_opt_defs[] = {
     "-cuda-lightweight",
     [] (gcore_params *params) { return &params->lightweight; },
     "Generate a lightweight coredump. This is equivalent to:\n" \
-      "--cuda-flags skip_nonrelocated_elf_images,skip_global_memory,skip_shared_memory,skip_local_memory"
+      "--cuda-flags skip_nonrelocated_elf_images,skip_global_memory,skip_shared_memory,skip_local_memory,skip_constbank_memory"
   },
 
   string_option_def<gcore_params> {
@@ -188,7 +188,8 @@ static const option_def gcore_opt_defs[] = {
       "\tskip_nonrelocated_elf_images\n"                                  \
       "\tskip_global_memory\n"                                            \
       "\tskip_shared_memory\n"                                            \
-      "\tskip_local_memory"
+      "\tskip_local_memory\n"                                             \
+      "\tskip_constbank_memory"
   }
 };
 
@@ -236,6 +237,8 @@ static CUDBGCoredumpGenerationFlags parse_cuda_core_flags(std::string& flags)
       result |= CUDBG_COREDUMP_SKIP_SHARED_MEMORY;
     else if (!flag.compare ("skip_local_memory"))
       result |= CUDBG_COREDUMP_SKIP_LOCAL_MEMORY;
+    else if (!flag.compare ("skip_constbank_memory"))
+      result |= CUDBG_COREDUMP_SKIP_CONSTBANK_MEMORY;
     else
       error (_("Unknown CUDA core dump flag: %s"), flag.c_str ());
   } while (pos != std::string::npos);
@@ -251,9 +254,10 @@ gcore_command (const char *args, int from_tty)
   gcore_params params{};
   const auto def_grp = make_gcore_opt_def_group (&params);
 
-  const char *curr = args;
-  if (curr && !process_options (&curr, PROCESS_OPTIONS_UNKNOWN_IS_ERROR, def_grp))
-    params.cpu_core_file = curr;
+  process_options (&args, PROCESS_OPTIONS_UNKNOWN_IS_OPERAND, def_grp);
+  /* CPU corefile name always is the last operand */
+  if (args && *args != '\0')
+    params.cpu_core_file = args;
 
   if (!params.no_gpu)
   {
@@ -305,6 +309,12 @@ gcore_command (const char *args, int from_tty)
   if (info_verbose)
     gdb_printf ("Opening corefile '%s' for output.\n",
 		corefilename.get ());
+#ifdef NVIDIA_CUDA_GDB
+  /* Ensure we switch back to host thread */
+  scoped_restore_current_thread restore_thread;
+  if (has_inferior_thread ())
+    switch_to_thread (inferior_thread ());
+#endif
 
   if (target_supports_dumpcore ())
     target_dumpcore (corefilename.get ());
@@ -569,10 +579,9 @@ gcore_create_callback (CORE_ADDR vaddr, unsigned long size, int read,
     {
       /* See if this region of memory lies inside a known file on disk.
 	 If so, we can avoid copying its contents by clearing SEC_LOAD.  */
-      struct obj_section *objsec;
 
       for (objfile *objfile : current_program_space->objfiles ())
-	ALL_OBJFILE_OSECTIONS (objfile, objsec)
+	for (obj_section *objsec : objfile->sections ())
 	  {
 	    bfd *abfd = objfile->obfd.get ();
 	    asection *asec = objsec->the_bfd_section;
@@ -676,12 +685,11 @@ objfile_find_memory_regions (struct target_ops *self,
 			     find_memory_region_ftype func, void *obfd)
 {
   /* Use objfile data to create memory sections.  */
-  struct obj_section *objsec;
   bfd_vma temp_bottom, temp_top;
 
   /* Call callback function for each objfile section.  */
   for (objfile *objfile : current_program_space->objfiles ())
-    ALL_OBJFILE_OSECTIONS (objfile, objsec)
+    for (obj_section *objsec : objfile->sections ())
       {
 	asection *isec = objsec->the_bfd_section;
 	flagword flags = bfd_section_flags (isec);
