@@ -17,6 +17,11 @@
    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.  */
 
+/* NVIDIA CUDA Debugger CUDA-GDB
+   Copyright (C) 2007-2025 NVIDIA Corporation
+   Modified from the original GDB file referenced above by the CUDA-GDB
+   team at NVIDIA <cudatools@nvidia.com>. */
+
 #ifndef PYTHON_PYTHON_INTERNAL_H
 #define PYTHON_PYTHON_INTERNAL_H
 
@@ -112,6 +117,7 @@
 typedef PY_LONG_LONG gdb_py_longest;
 typedef unsigned PY_LONG_LONG gdb_py_ulongest;
 #define gdb_py_long_as_ulongest PyLong_AsUnsignedLongLong
+#define gdb_py_long_as_long_and_overflow PyLong_AsLongLongAndOverflow
 
 #else /* HAVE_LONG_LONG */
 
@@ -120,6 +126,7 @@ typedef unsigned PY_LONG_LONG gdb_py_ulongest;
 typedef long gdb_py_longest;
 typedef unsigned long gdb_py_ulongest;
 #define gdb_py_long_as_ulongest PyLong_AsUnsignedLong
+#define gdb_py_long_as_long_and_overflow PyLong_AsLongAndOverflow
 
 #endif /* HAVE_LONG_LONG */
 
@@ -167,7 +174,6 @@ GDBDYN_EXTERN PyObject *gdbdyn_None;
 GDBDYN_EXTERN PyObject *gdbdyn_True;
 GDBDYN_EXTERN PyObject *gdbdyn_False;
 GDBDYN_EXTERN PyObject *gdbdyn_NotImplemented;
-
 GDBDYN_EXTERN PyTypeObject *gdbdyn_FloatType;
 GDBDYN_EXTERN PyTypeObject *gdbdyn_BoolType;
 GDBDYN_EXTERN PyTypeObject *gdbdyn_SliceType;
@@ -180,6 +186,7 @@ GDBDYN_EXTERN PyObject **pgdbpyExc_IOError;
 GDBDYN_EXTERN PyObject **pgdbpyExc_KeyError;
 GDBDYN_EXTERN PyObject **pgdbpyExc_KeyboardInterrupt;
 GDBDYN_EXTERN PyObject **pgdbpyExc_MemoryError;
+GDBDYN_EXTERN PyObject **pgdbpyExc_NameError;
 GDBDYN_EXTERN PyObject **pgdbpyExc_NotImplementedError;
 GDBDYN_EXTERN PyObject **pgdbpyExc_OverflowError;
 GDBDYN_EXTERN PyObject **pgdbpyExc_RuntimeError;
@@ -187,7 +194,6 @@ GDBDYN_EXTERN PyObject **pgdbpyExc_StopIteration;
 GDBDYN_EXTERN PyObject **pgdbpyExc_SystemError;
 GDBDYN_EXTERN PyObject **pgdbpyExc_TypeError;
 GDBDYN_EXTERN PyObject **pgdbpyExc_ValueError;
-GDBDYN_EXTERN PyObject **pgdbpyExc_NameError;
 
 GDBDYN_EXTERN PyThreadState **pgdbdyn_OSReadlineTState;
 
@@ -197,6 +203,7 @@ GDBDYN_EXTERN char *(**pgdbdyn_PyOS_ReadlineFunctionPointer) (FILE*, FILE*, cons
 /* Dynamic reference to functions */
 GDBDYN_EXTERN int (*gdbdyn_Arg_UnpackTuple)(PyObject *, const char *, Py_ssize_t, Py_ssize_t, ...);
 GDBDYN_EXTERN PyObject * (*gdbdyn_ErrFormat) (PyObject *, const char *, ...);
+GDBDYN_EXTERN PyObject * (*gdbdyn_FatalError) (const char *);
 GDBDYN_EXTERN PyObject * (*gdbdyn_BuildValue) (const char *, ...);
 GDBDYN_EXTERN PyObject * (*gdbdyn_PyObject_CallFunctionObjArgs) (PyObject *,...);
 GDBDYN_EXTERN PyObject * (*gdbdyn_PyObject_CallMethodObjArgs) (PyObject *, PyObject *,...);
@@ -259,6 +266,7 @@ GDBDYN_EXTERN int (*gdbdyn_PyList_Insert) (PyObject *, Py_ssize_t, PyObject *);
 GDBDYN_EXTERN PyObject * (*gdbdyn_PyList_New) (Py_ssize_t size);
 GDBDYN_EXTERN Py_ssize_t (*gdbdyn_PyList_Size) (PyObject *);
 GDBDYN_EXTERN PY_LONG_LONG (*gdbdyn_PyLong_AsLongLong) (PyObject *);
+GDBDYN_EXTERN PY_LONG_LONG (*gdbdyn_PyLong_AsLongLongAndOverflow) (PyObject *, int *);
 GDBDYN_EXTERN long (*gdbdyn_PyLong_AsLong) (PyObject *);
 GDBDYN_EXTERN unsigned PY_LONG_LONG (*gdbdyn_PyLong_AsUnsignedLongLong) (PyObject *);
 GDBDYN_EXTERN Py_ssize_t (*gdbdyn_PyLong_AsSsize_t) (PyObject *pylong);
@@ -361,6 +369,7 @@ GDBDYN_EXTERN void (*gdbdyn_Py_DecRef) (PyObject *);
 #define PyExc_KeyError (*pgdbpyExc_KeyError)
 #define PyExc_KeyboardInterrupt (*pgdbpyExc_KeyboardInterrupt)
 #define PyExc_MemoryError (*pgdbpyExc_MemoryError)
+#define PyExc_NameError (*pgdbpyExc_NameError) 
 #define PyExc_NotImplementedError (*pgdbpyExc_NotImplementedError)
 #define PyExc_OverflowError (*pgdbpyExc_OverflowError)
 #define PyExc_RuntimeError (*pgdbpyExc_RuntimeError)
@@ -368,7 +377,6 @@ GDBDYN_EXTERN void (*gdbdyn_Py_DecRef) (PyObject *);
 #define PyExc_SystemError (*pgdbpyExc_SystemError)
 #define PyExc_TypeError (*pgdbpyExc_TypeError)
 #define PyExc_ValueError (*pgdbpyExc_ValueError)
-#define PyExc_NameError (*pgdbpyExc_NameError) 
 
 #define _PyOS_ReadlineTState (*pgdbdyn_OSReadlineTState)
 
@@ -400,10 +408,12 @@ gdbdyn_pywrapper (Args&&... args)
 /* Redefine python functions */
 #undef PyArg_UnpackTuple
 #define PyArg_UnpackTuple GDBDYN_PYWRAPPER(gdbdyn_Arg_UnpackTuple)
-#undef PyErr_Format
-#define PyErr_Format GDBDYN_PYWRAPPER(gdbdyn_ErrFormat)
 #undef Py_BuildValue
 #define Py_BuildValue GDBDYN_PYWRAPPER(gdbdyn_BuildValue)
+#undef Py_FatalError
+#define Py_FatalError GDBDYN_PYWRAPPER(gdbdyn_FatalError)
+#undef PyErr_Format
+#define PyErr_Format GDBDYN_PYWRAPPER(gdbdyn_ErrFormat)
 #undef PyObject_CallFunctionObjArgs
 #define PyObject_CallFunctionObjArgs GDBDYN_PYWRAPPER(gdbdyn_PyObject_CallFunctionObjArgs)
 #undef PyObject_CallMethodObjArgs
@@ -526,6 +536,8 @@ gdbdyn_pywrapper (Args&&... args)
 #define PyList_Size GDBDYN_PYWRAPPER(gdbdyn_PyList_Size)
 #undef PyLong_AsLongLong
 #define PyLong_AsLongLong GDBDYN_PYWRAPPER(gdbdyn_PyLong_AsLongLong)
+#undef PyLong_AsLongLongAndOverflow
+#define PyLong_AsLongLongAndOverflow GDBDYN_PYWRAPPER(gdbdyn_PyLong_AsLongLongAndOverflow)
 #undef PyLong_AsLong
 #define PyLong_AsLong GDBDYN_PYWRAPPER(gdbdyn_PyLong_AsLong)
 #undef PyLong_AsUnsignedLongLong
@@ -1064,7 +1076,6 @@ PyObject *symbol_to_symbol_object (struct symbol *sym);
 PyObject *block_to_block_object (const struct block *block,
 				 struct objfile *objfile);
 PyObject *value_to_value_object (struct value *v);
-PyObject *value_to_value_object_no_release (struct value *v);
 PyObject *type_to_type_object (struct type *);
 PyObject *frame_info_to_frame_object (frame_info_ptr frame);
 PyObject *symtab_to_linetable_object (PyObject *symtab);
@@ -1109,6 +1120,9 @@ struct symtab_and_line *sal_object_to_symtab_and_line (PyObject *obj);
 frame_info_ptr frame_object_to_frame_info (PyObject *frame_obj);
 struct gdbarch *arch_object_to_gdbarch (PyObject *obj);
 
+extern PyObject *gdbpy_execute_mi_command (PyObject *self, PyObject *args,
+					   PyObject *kw);
+
 /* Convert Python object OBJ to a program_space pointer.  OBJ must be a
    gdb.Progspace reference.  Return nullptr if the gdb.Progspace is not
    valid (see gdb.Progspace.is_valid), otherwise return the program_space
@@ -1116,76 +1130,120 @@ struct gdbarch *arch_object_to_gdbarch (PyObject *obj);
 
 extern struct program_space *progspace_object_to_program_space (PyObject *obj);
 
-void gdbpy_initialize_gdb_readline (void);
-int gdbpy_initialize_auto_load (void)
-  CPYCHECKER_NEGATIVE_RESULT_SETS_EXCEPTION;
-int gdbpy_initialize_values (void)
-  CPYCHECKER_NEGATIVE_RESULT_SETS_EXCEPTION;
-int gdbpy_initialize_frames (void)
-  CPYCHECKER_NEGATIVE_RESULT_SETS_EXCEPTION;
-int gdbpy_initialize_instruction (void)
-  CPYCHECKER_NEGATIVE_RESULT_SETS_EXCEPTION;
-int gdbpy_initialize_btrace (void)
-  CPYCHECKER_NEGATIVE_RESULT_SETS_EXCEPTION;
-int gdbpy_initialize_record (void)
-  CPYCHECKER_NEGATIVE_RESULT_SETS_EXCEPTION;
-int gdbpy_initialize_symtabs (void)
-  CPYCHECKER_NEGATIVE_RESULT_SETS_EXCEPTION;
-int gdbpy_initialize_commands (void)
-  CPYCHECKER_NEGATIVE_RESULT_SETS_EXCEPTION;
-int gdbpy_initialize_symbols (void)
-  CPYCHECKER_NEGATIVE_RESULT_SETS_EXCEPTION;
-int gdbpy_initialize_symtabs (void)
-  CPYCHECKER_NEGATIVE_RESULT_SETS_EXCEPTION;
-int gdbpy_initialize_blocks (void)
-  CPYCHECKER_NEGATIVE_RESULT_SETS_EXCEPTION;
-int gdbpy_initialize_types (void)
-  CPYCHECKER_NEGATIVE_RESULT_SETS_EXCEPTION;
-int gdbpy_initialize_functions (void)
-  CPYCHECKER_NEGATIVE_RESULT_SETS_EXCEPTION;
-int gdbpy_initialize_pspace (void)
-  CPYCHECKER_NEGATIVE_RESULT_SETS_EXCEPTION;
-int gdbpy_initialize_objfile (void)
-  CPYCHECKER_NEGATIVE_RESULT_SETS_EXCEPTION;
-int gdbpy_initialize_breakpoints (void)
-  CPYCHECKER_NEGATIVE_RESULT_SETS_EXCEPTION;
-int gdbpy_initialize_breakpoint_locations ()
-  CPYCHECKER_NEGATIVE_RESULT_SETS_EXCEPTION;
-int gdbpy_initialize_finishbreakpoints (void)
-  CPYCHECKER_NEGATIVE_RESULT_SETS_EXCEPTION;
-int gdbpy_initialize_lazy_string (void)
-  CPYCHECKER_NEGATIVE_RESULT_SETS_EXCEPTION;
-int gdbpy_initialize_linetable (void)
-  CPYCHECKER_NEGATIVE_RESULT_SETS_EXCEPTION;
-int gdbpy_initialize_parameters (void)
-  CPYCHECKER_NEGATIVE_RESULT_SETS_EXCEPTION;
-int gdbpy_initialize_thread (void)
-  CPYCHECKER_NEGATIVE_RESULT_SETS_EXCEPTION;
-int gdbpy_initialize_inferior (void)
-  CPYCHECKER_NEGATIVE_RESULT_SETS_EXCEPTION;
-int gdbpy_initialize_eventregistry (void)
-  CPYCHECKER_NEGATIVE_RESULT_SETS_EXCEPTION;
-int gdbpy_initialize_event (void)
-  CPYCHECKER_NEGATIVE_RESULT_SETS_EXCEPTION;
-int gdbpy_initialize_arch (void)
-  CPYCHECKER_NEGATIVE_RESULT_SETS_EXCEPTION;
-int gdbpy_initialize_registers ()
-  CPYCHECKER_NEGATIVE_RESULT_SETS_EXCEPTION;
-int gdbpy_initialize_xmethods (void)
-  CPYCHECKER_NEGATIVE_RESULT_SETS_EXCEPTION;
-int gdbpy_initialize_unwind (void)
-  CPYCHECKER_NEGATIVE_RESULT_SETS_EXCEPTION;
-int gdbpy_initialize_tui ()
-  CPYCHECKER_NEGATIVE_RESULT_SETS_EXCEPTION;
-int gdbpy_initialize_membuf ()
-  CPYCHECKER_NEGATIVE_RESULT_SETS_EXCEPTION;
-int gdbpy_initialize_connection ()
-  CPYCHECKER_NEGATIVE_RESULT_SETS_EXCEPTION;
-int gdbpy_initialize_micommands (void)
-  CPYCHECKER_NEGATIVE_RESULT_SETS_EXCEPTION;
-void gdbpy_finalize_micommands ();
-int gdbpy_initialize_disasm ()
-  CPYCHECKER_NEGATIVE_RESULT_SETS_EXCEPTION;
+/* A class for managing the initialization, and finalization functions
+   from all Python files (e.g. gdb/python/py-*.c).
+
+   Within any Python file, create an instance of this class, passing in
+   the initialization function, and, optionally, the finalization
+   function.
+
+   These functions are added to a single global list of functions, which
+   can then be called from do_start_initialization and finalize_python
+   (see python.c) to initialize all the Python files within GDB.  */
+
+class gdbpy_initialize_file
+{
+  /* The type of a function that can be called just after GDB has setup the
+     Python interpreter.  This function will setup any additional Python
+     state required by a particular subsystem.  Return 0 if the setup was
+     successful, or return -1 if setup failed, in which case a Python
+     exception should have been raised.  */
+
+  using gdbpy_initialize_file_ftype = int (*) (void);
+
+  /* The type of a function that can be called just before GDB shuts down
+     the Python interpreter.  This function can cleanup an Python state
+     that is cached within GDB, for example, if GDB is holding any
+     references to Python objects, these should be released before the
+     Python interpreter is shut down.
+
+     There is no error return in this case.  This function is only called
+     when GDB is already shutting down.  The function should make a best
+     effort to clean up, and then return.  */
+
+  using gdbpy_finalize_file_ftype = void (*) (void);
+
+  /* The type for an initialization and finalization function pair.  */
+
+  using callback_pair_t = std::pair<gdbpy_initialize_file_ftype,
+				    gdbpy_finalize_file_ftype>;
+
+  /* Return the vector of callbacks.  The vector is defined as a static
+     variable within this function so that it will be initialized the first
+     time this function is called.  This is important, as this function is
+     called as part of the global object initialization process; if the
+     vector was a static variable within this class then we could not
+     guarantee that it had been initialized before it was used.  */
+
+  static std::vector<callback_pair_t> &
+  callbacks ()
+  {
+    static std::vector<callback_pair_t> list;
+    return list;
+  }
+
+public:
+
+  /* Register the initialization (INIT) and finalization (FINI) functions
+     for a Python file.  See the comments on the function types above for
+     when these functions will be called.
+
+     Either of these functions can be nullptr, in which case no function
+     will be called.
+
+     The FINI argument is optional, and defaults to nullptr (no function to
+     call).  */
+
+  gdbpy_initialize_file (gdbpy_initialize_file_ftype init,
+			 gdbpy_finalize_file_ftype fini = nullptr)
+  {
+    callbacks ().emplace_back (init, fini);
+  }
+
+  /* Run all the Python file initialize functions and return true.  If any
+     of the initialize functions fails then this function returns false.
+     In the case of failure it is undefined how many of the initialize
+     functions will have been called.  */
+
+  static bool
+  initialize_all ()
+  {
+    /* The initialize_all function should only be called once.  The
+       following check reverses the global list, which will effect this
+       initialize_all call, as well as the later finalize_all call.
+
+       The environment variable checked here is the same as the one checked
+       in the generated init.c file.  */
+    if (getenv ("GDB_REVERSE_INIT_FUNCTIONS") != nullptr)
+      std::reverse (callbacks ().begin (), callbacks ().end ());
+
+    for (const auto &p : gdbpy_initialize_file::callbacks ())
+      {
+	if (p.first != nullptr && p.first () < 0)
+	  return false;
+      }
+    return true;
+  }
+
+  /* Run all the Python file finalize functions.  */
+
+  static void
+  finalize_all ()
+  {
+    for (const auto &p : gdbpy_initialize_file::callbacks ())
+      {
+	if (p.second != nullptr)
+	  p.second ();
+      }
+  }
+};
+
+/* Macro to simplify registering the initialization and finalization
+   functions for a Python file.  */
+
+#define GDBPY_INITIALIZE_FILE(INIT, ...)				\
+  static gdbpy_initialize_file						\
+    CONCAT(gdbpy_initialize_file_obj_, __LINE__) (INIT, ##__VA_ARGS__)
 
 PyMODINIT_FUNC gdbpy_events_mod_func ();
 
@@ -1404,6 +1462,7 @@ extern const struct value_print_options *gdbpy_current_print_options;
 
 void bpfinishpy_pre_stop_hook (struct gdbpy_breakpoint_object *bp_obj);
 void bpfinishpy_post_stop_hook (struct gdbpy_breakpoint_object *bp_obj);
+void bpfinishpy_pre_delete_hook (struct gdbpy_breakpoint_object *bp_obj);
 
 extern PyObject *gdbpy_doc_cst;
 extern PyObject *gdbpy_children_cst;
