@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2014-2024 NVIDIA CORPORATION. All rights reserved.
+ * Copyright (c) 2014-2025 NVIDIA CORPORATION. All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -1188,6 +1188,8 @@ static int cuCoreProcessSection(CudaCore *cc, bool *processed, Elf_Scn *scn)
 		return cuCoreReadWarpInfo("uregs", cc, scn);
 	case CUDBG_SHT_DEV_UPRED:
 		return cuCoreReadWarpInfo("upred", cc, scn);
+	case CUDBG_SHT_CBU_BAR:
+		return cuCoreReadWarpInfo("cbu_bar", cc, scn);
 	case CUDBG_SHT_CB_TABLE:
 		return cuCoreReadConstBanksTable(cc, scn);
 	default:
