@@ -904,6 +904,7 @@ struct __CUDA_ALIGN__(2) __nv_fp4x4_e2m1 {
      * Constructor from \p double4 vector data type,
      * relies on \p __NV_SATFINITE behavior for out-of-range values.
      */
+    __NV_SILENCE_DEPRECATION_BEGIN
     explicit __CUDA_HOSTDEVICE_FP4__ __nv_fp4x4_e2m1(const double4 f) {
         const double2 flo = {f.x, f.y};
         const double2 fhi = {f.z, f.w};
@@ -913,6 +914,38 @@ struct __CUDA_ALIGN__(2) __nv_fp4x4_e2m1 {
             __nv_cvt_double2_to_fp4x2(fhi, __NV_E2M1, cudaRoundNearest);
         __x = __internal_pack_u8x2_to_u16(rlo, rhi);
     }
+    __NV_SILENCE_DEPRECATION_END
+
+    /**
+     * \ingroup CUDA_MATH_FP4_MISC
+     * Constructor from \p double4_16a vector data type,
+     * relies on \p __NV_SATFINITE behavior for out-of-range values.
+     */
+    explicit __CUDA_HOSTDEVICE_FP4__ __nv_fp4x4_e2m1(const double4_16a f) {
+        const double2 flo = {f.x, f.y};
+        const double2 fhi = {f.z, f.w};
+        const __nv_fp4x2_storage_t rlo =
+            __nv_cvt_double2_to_fp4x2(flo, __NV_E2M1, cudaRoundNearest);
+        const __nv_fp4x2_storage_t rhi =
+            __nv_cvt_double2_to_fp4x2(fhi, __NV_E2M1, cudaRoundNearest);
+        __x = __internal_pack_u8x2_to_u16(rlo, rhi);
+    }
+
+    /**
+     * \ingroup CUDA_MATH_FP4_MISC
+     * Constructor from \p double4_32a vector data type,
+     * relies on \p __NV_SATFINITE behavior for out-of-range values.
+     */
+    explicit __CUDA_HOSTDEVICE_FP4__ __nv_fp4x4_e2m1(const double4_32a f) {
+        const double2 flo = {f.x, f.y};
+        const double2 fhi = {f.z, f.w};
+        const __nv_fp4x2_storage_t rlo =
+            __nv_cvt_double2_to_fp4x2(flo, __NV_E2M1, cudaRoundNearest);
+        const __nv_fp4x2_storage_t rhi =
+            __nv_cvt_double2_to_fp4x2(fhi, __NV_E2M1, cudaRoundNearest);
+        __x = __internal_pack_u8x2_to_u16(rlo, rhi);
+    }
+    
 
 #if !defined(__CUDA_NO_FP4_CONVERSION_OPERATORS__)
     /* Widening converts */
