@@ -437,6 +437,9 @@ struct cudaOccFuncAttributes {
                             // function.
 
     int numBlockBarriers;   // Number of block barriers used (default to 1)
+
+    int virtualResourceCount;
+                            // Amount of virtual resources used.
 #ifdef __cplusplus
     // This structure can be converted from a cudaFuncAttributes structure for
     // users that use this header in their CUDA applications.
@@ -474,7 +477,8 @@ struct cudaOccFuncAttributes {
         partitionedGCConfig (PARTITIONED_GC_OFF),
         shmemLimitConfig    (FUNC_SHMEM_LIMIT_OPTIN),
         maxDynamicSharedSizeBytes (attr.maxDynamicSharedSizeBytes),
-        numBlockBarriers    (1)
+        numBlockBarriers    (1),
+        virtualResourceCount (0)
     {}
 
     __OCC_INLINE
@@ -485,7 +489,8 @@ struct cudaOccFuncAttributes {
         partitionedGCConfig (PARTITIONED_GC_OFF),
         shmemLimitConfig    (FUNC_SHMEM_LIMIT_DEFAULT),
         maxDynamicSharedSizeBytes (0),
-        numBlockBarriers    (0)
+        numBlockBarriers    (0),
+        virtualResourceCount (0)
     {}
 #endif
 };
@@ -531,7 +536,8 @@ typedef enum cudaOccLimitingFactor_enum {
     OCC_LIMIT_REGISTERS     = 0x02, // - registers available
     OCC_LIMIT_SHARED_MEMORY = 0x04, // - shared memory available
     OCC_LIMIT_BLOCKS        = 0x08, // - blocks available
-    OCC_LIMIT_BARRIERS      = 0x10  // - barrier available
+    OCC_LIMIT_BARRIERS      = 0x10,  // - barrier available
+    OCC_LIMIT_VIRTUAL_RESOURCES = 0x20  // - virtual resources available
 } cudaOccLimitingFactor;
 
 /**
@@ -554,6 +560,7 @@ struct cudaOccResult {
     int blockLimitBlocks;              // Occupancy due to maximum number of blocks
                                        // managable per SM
     int blockLimitBarriers;            // Occupancy due to block barrier usage
+    int blockLimitVirtual;             // Occupancy due to virtual resources
     int allocatedRegistersPerBlock;    // Actual number of registers allocated per
                                        // block
     size_t allocatedSharedMemPerBlock; // Actual size of shared memory allocated
@@ -624,6 +631,7 @@ static __OCC_INLINE cudaOccError cudaOccSMemAllocationGranularity(int *limit, co
         case 8:
         case 9:
         case 10:
+        case 11:
         case 12:
             value = 128;
             break;
@@ -653,6 +661,7 @@ static __OCC_INLINE cudaOccError cudaOccRegAllocationMaxPerThread(int *limit, co
         case 8:
         case 9:
         case 10:
+        case 11:
         case 12:
             value = 256;
             break;
@@ -680,6 +689,7 @@ static __OCC_INLINE cudaOccError cudaOccRegAllocationGranularity(int *limit, con
         case 8:
         case 9:
         case 10:
+        case 11:
         case 12:
             value = 256;
             break;
@@ -706,6 +716,7 @@ static __OCC_INLINE cudaOccError cudaOccSubPartitionsPerMultiprocessor(int *limi
         case 8:
         case 9:
         case 10:
+        case 11:
         case 12:
             value = 4;
             break;
@@ -766,6 +777,15 @@ static __OCC_INLINE cudaOccError cudaOccMaxBlocksPerMultiprocessor(int* limit, c
                     value = 32;
             }
             break;
+        case 11:
+            switch(properties->computeMinor) {
+                case 0 :
+                    value = 24;
+                    break;
+                default :
+                    value = 32;
+            }
+            break;
         case 12:
             value = 24;
             break;
@@ -919,7 +939,8 @@ static __OCC_INLINE cudaOccError cudaOccAlignUpShmemSizeVoltaPlus(size_t *shMemS
         }
         break;
     }
-    case 10: {
+    case 10:
+    case 11: {
         switch (properties->computeMinor) {
     // GB10x GPUs in Blackwell family have the below compute minors and corresponding
     // shared memory configs
@@ -1136,6 +1157,7 @@ static __OCC_INLINE cudaOccError cudaOccSMemPerBlock(size_t *limit, const cudaOc
         case 8:
         case 9:
         case 10:
+        case 11:
         case 12:
             switch (shmemLimitConfig) {
                 default:
@@ -1183,6 +1205,28 @@ static __OCC_INLINE cudaOccError cudaOccPartitionedGlobalCachingModeSupport(cuda
     return CUDA_OCC_SUCCESS;
 }
 
+/**
+ * Max virtual resource count per SM
+ */
+static __OCC_INLINE cudaOccError cudaOccMaxVirtualResourceCountPerSM(int *count, const cudaOccDeviceProp *properties)
+{
+    *count = 0;
+
+    switch (properties->computeMajor) {
+        case 10:
+        case 11:
+        case 12:
+            *count = 128;
+            break;
+        default:
+            *count = 0;
+            break;
+    }
+
+    return CUDA_OCC_SUCCESS;
+}
+
+
 ///////////////////////////////////////////////
 //            User Input Sanity              //
 ///////////////////////////////////////////////
@@ -1594,6 +1638,16 @@ static __OCC_INLINE cudaOccError cudaOccMaxBlocksPerSMBlockBarrierLimit(
                     numBarriersAvailable = ctaLimitBlocks * 2;
             }
 
+            break;
+        case 11:
+            switch(properties->computeMinor) {
+                case 0 :
+                    numBarriersAvailable = ctaLimitBlocks;
+                    break;
+                default :
+                    numBarriersAvailable = ctaLimitBlocks * 2;
+            }
+
             break;
         case 12:
             numBarriersAvailable = ctaLimitBlocks;
@@ -1611,6 +1665,41 @@ static __OCC_INLINE cudaOccError cudaOccMaxBlocksPerSMBlockBarrierLimit(
     return status;
 }
 
+/**
+ * Compute occupancy based on virtual resource count
+ */
+static __OCC_INLINE cudaOccError cudaOccMaxBlocksPerSMVirtualResource(
+    int                          *limit,
+    const cudaOccDeviceProp     *properties,
+    const cudaOccFuncAttributes *attributes)
+{
+    cudaOccError status = CUDA_OCC_SUCCESS;
+    int maxVirtualResourcePerSM = 0;
+    int maxBlocks = INT_MAX;
+
+    // Get the maximum virtual resource count per SM for this device
+    status = cudaOccMaxVirtualResourceCountPerSM(&maxVirtualResourcePerSM, properties);
+    if (status != CUDA_OCC_SUCCESS) {
+        return status;
+    }
+
+    // If the device doesn't support virtual resources or the kernel doesn't use them,
+    // this limit doesn't apply
+    if (maxVirtualResourcePerSM <= 0 || attributes->virtualResourceCount <= 0) {
+        *limit = INT_MAX;
+        return CUDA_OCC_SUCCESS;
+    }
+
+    // Calculate the maximum number of blocks based on virtual resource usage
+    maxBlocks = maxVirtualResourcePerSM / attributes->virtualResourceCount;
+    
+    *limit = maxBlocks;
+
+    return CUDA_OCC_SUCCESS;
+}
+
+
+
 ///////////////////////////////////
 //      API Implementations      //
 ///////////////////////////////////
@@ -1630,6 +1719,7 @@ cudaOccError cudaOccMaxActiveBlocksPerMultiprocessor(
     int          ctaLimitSMem    = 0;
     int          ctaLimitRegs    = 0;
     int          ctaLimitBars    = 0;
+    int          ctaLimitVirtual = 0;
     int          ctaLimit        = 0;
     unsigned int limitingFactors = 0;
     
@@ -1712,13 +1802,20 @@ cudaOccError cudaOccMaxActiveBlocksPerMultiprocessor(
         return status;
     }
 
+    // Limits due to virtual resources/SM
+    //
+    status = cudaOccMaxBlocksPerSMVirtualResource(&ctaLimitVirtual, properties, attributes);
+    if (status != CUDA_OCC_SUCCESS) {
+        return status;
+    }
+
     ///////////////////////////
     // Overall occupancy
     ///////////////////////////
 
     // Overall limit is min() of limits due to above reasons
     //
-    ctaLimit = __occMin(ctaLimitRegs, __occMin(ctaLimitSMem, __occMin(ctaLimitWarps, ctaLimitBlocks)));
+    ctaLimit = __occMin(ctaLimitRegs, __occMin(ctaLimitSMem, __occMin(ctaLimitWarps, __occMin(ctaLimitBlocks, ctaLimitVirtual))));
 
     // Determine occupancy limiting factors
     //
@@ -1734,6 +1831,9 @@ cudaOccError cudaOccMaxActiveBlocksPerMultiprocessor(
     if (ctaLimit == ctaLimitBlocks) {
         limitingFactors |= OCC_LIMIT_BLOCKS;
     }
+    if (ctaLimit == ctaLimitVirtual) {
+        limitingFactors |= OCC_LIMIT_VIRTUAL_RESOURCES;
+    }
 
     // For Hopper onwards compute the limits to occupancy based on block barrier count
     //
@@ -1768,6 +1868,7 @@ cudaOccError cudaOccMaxActiveBlocksPerMultiprocessor(
     result->blockLimitWarps     = ctaLimitWarps;
     result->blockLimitBlocks    = ctaLimitBlocks;
     result->blockLimitBarriers  = ctaLimitBars;
+    result->blockLimitVirtual   = ctaLimitVirtual;
     result->partitionedGCConfig = gcConfig;
 
     // Final occupancy
