@@ -1,7 +1,7 @@
 /*
  * NVIDIA_COPYRIGHT_BEGIN
  *
- * Copyright (c) 2022-2024, NVIDIA CORPORATION.  All rights reserved.
+ * Copyright (c) 2022-2025, NVIDIA CORPORATION.  All rights reserved.
  *
  * NVIDIA CORPORATION and its licensors retain all intellectual property
  * and proprietary rights in and to this software, related documentation
@@ -45,25 +45,17 @@ typedef enum {
   NVJITLINK_ERROR_THREADPOOL, /**< Issue with Thread Pool */
   NVJITLINK_ERROR_UNRECOGNIZED_INPUT, /**< Unrecognized Input */
   NVJITLINK_ERROR_FINALIZE, /**< Finalizer Error */
-#ifdef NEW_ERROR_CODES // These error codes will appear in a future CUDA release.
-  NVJITLINK_ERROR_NULL_INPUT,
-  NVJITLINK_ERROR_INCOMPATIBLE_OPTIONS,
-  NVJITLINK_ERROR_INCORRECT_INPUT_TYPE,
-  NVJITLINK_ERROR_ARCH_MISMATCH,
-  NVJITLINK_ERROR_OUTDATED_LIBRARY,
-  NVJITLINK_ERROR_MISSING_FATBIN
-#endif
+  NVJITLINK_ERROR_NULL_INPUT, /**< Null Input */
+  NVJITLINK_ERROR_INCOMPATIBLE_OPTIONS, /**< Incompatible Options */
+  NVJITLINK_ERROR_INCORRECT_INPUT_TYPE, /**< Incorrect Input Type */
+  NVJITLINK_ERROR_ARCH_MISMATCH, /**< Arch Mismatch */
+  NVJITLINK_ERROR_OUTDATED_LIBRARY, /**< Outdated Library */
+  NVJITLINK_ERROR_MISSING_FATBIN, /**< Missing Fatbin */
+  NVJITLINK_ERROR_UNRECOGNIZED_ARCH, /**< Unrecognized -arch value */
+  NVJITLINK_ERROR_UNSUPPORTED_ARCH, /**< Unsupported -arch value */
+  NVJITLINK_ERROR_LTO_NOT_ENABLED /**< Requires -lto */
 } nvJitLinkResult;
 
-#ifndef NEW_ERROR_CODES // To avoid breaking compatibility, we map them to existing error codes for now.
-#define NVJITLINK_ERROR_NULL_INPUT NVJITLINK_ERROR_INVALID_INPUT
-#define NVJITLINK_ERROR_INCOMPATIBLE_OPTIONS NVJITLINK_ERROR_INVALID_INPUT
-#define NVJITLINK_ERROR_INCORRECT_INPUT_TYPE NVJITLINK_ERROR_INVALID_INPUT
-#define NVJITLINK_ERROR_ARCH_MISMATCH NVJITLINK_ERROR_INTERNAL
-#define NVJITLINK_ERROR_OUTDATED_LIBRARY NVJITLINK_ERROR_INTERNAL
-#define NVJITLINK_ERROR_MISSING_FATBIN NVJITLINK_ERROR_INVALID_INPUT
-#endif  
-
 /**
  *
  * \defgroup linking Linking
@@ -171,10 +163,10 @@ typedef struct nvJitLink* nvJitLinkHandle; // opaque handle
 
 // For versioning we will have separate API version for each library version
 
-extern nvJitLinkResult __nvJitLinkCreate_12_9(
+extern nvJitLinkResult __nvJitLinkCreate_13_0(
   nvJitLinkHandle *handle,
   uint32_t numOptions,
-  const char **options);
+  const char * const * options);
 /**
  * \ingroup linking
  * \brief   nvJitLinkCreate creates an instance of nvJitLinkHandle with the
@@ -200,11 +192,11 @@ static inline nvJitLinkResult nvJitLinkCreate(
   uint32_t numOptions,
   const char **options)
 {
-  return __nvJitLinkCreate_12_9 (handle, numOptions, options);
+  return __nvJitLinkCreate_13_0 (handle, numOptions, options);
 }
 #endif
  
-extern nvJitLinkResult __nvJitLinkDestroy_12_9 (nvJitLinkHandle *handle);
+extern nvJitLinkResult __nvJitLinkDestroy_13_0 (nvJitLinkHandle *handle);
 /**
  * \ingroup linking
  * \brief   nvJitLinkDestroy frees the memory associated with the given handle
@@ -221,11 +213,11 @@ extern nvJitLinkResult __nvJitLinkDestroy_12_9 (nvJitLinkHandle *handle);
 #ifndef NVJITLINK_NO_INLINE
 static inline nvJitLinkResult nvJitLinkDestroy (nvJitLinkHandle *handle)
 {
-  return __nvJitLinkDestroy_12_9 (handle);
+  return __nvJitLinkDestroy_13_0 (handle);
 }
 #endif
  
-extern nvJitLinkResult __nvJitLinkAddData_12_9(
+extern nvJitLinkResult __nvJitLinkAddData_13_0(
   nvJitLinkHandle handle,
   nvJitLinkInputType inputType,
   const void *data,
@@ -253,11 +245,11 @@ static inline nvJitLinkResult nvJitLinkAddData(
   size_t size,
   const char *name) // name can be null
 {
-  return __nvJitLinkAddData_12_9 (handle, inputType, data, size, name);
+  return __nvJitLinkAddData_13_0 (handle, inputType, data, size, name);
 }
 #endif
  
-extern nvJitLinkResult __nvJitLinkAddFile_12_9(
+extern nvJitLinkResult __nvJitLinkAddFile_13_0(
   nvJitLinkHandle handle,
   nvJitLinkInputType inputType,
   const char *fileName); // includes path to file
@@ -279,11 +271,11 @@ static inline nvJitLinkResult nvJitLinkAddFile(
   nvJitLinkInputType inputType,
   const char *fileName) // includes path to file
 {
-  return __nvJitLinkAddFile_12_9 (handle, inputType, fileName);
+  return __nvJitLinkAddFile_13_0 (handle, inputType, fileName);
 }
 #endif
  
-extern nvJitLinkResult __nvJitLinkComplete_12_9 (nvJitLinkHandle handle);
+extern nvJitLinkResult __nvJitLinkComplete_13_0 (nvJitLinkHandle handle);
 /**
  * \ingroup linking
  * \brief   nvJitLinkComplete does the actual link. 
@@ -297,11 +289,11 @@ extern nvJitLinkResult __nvJitLinkComplete_12_9 (nvJitLinkHandle handle);
 #ifndef NVJITLINK_NO_INLINE
 static inline nvJitLinkResult nvJitLinkComplete (nvJitLinkHandle handle)
 {
-  return __nvJitLinkComplete_12_9 (handle);
+  return __nvJitLinkComplete_13_0 (handle);
 }
 #endif
  
-extern nvJitLinkResult __nvJitLinkGetLinkedCubinSize_12_9(
+extern nvJitLinkResult __nvJitLinkGetLinkedCubinSize_13_0(
   nvJitLinkHandle handle,
   size_t *size);
 /**
@@ -322,11 +314,11 @@ static inline nvJitLinkResult nvJitLinkGetLinkedCubinSize(
   nvJitLinkHandle handle,
   size_t *size)
 {
-  return __nvJitLinkGetLinkedCubinSize_12_9 (handle, size);
+  return __nvJitLinkGetLinkedCubinSize_13_0 (handle, size);
 }
 #endif
 
-extern nvJitLinkResult __nvJitLinkGetLinkedCubin_12_9(
+extern nvJitLinkResult __nvJitLinkGetLinkedCubin_13_0(
   nvJitLinkHandle handle,
   void *cubin);
 /**
@@ -348,11 +340,11 @@ static inline nvJitLinkResult nvJitLinkGetLinkedCubin(
   nvJitLinkHandle handle,
   void *cubin)
 {
-  return __nvJitLinkGetLinkedCubin_12_9 (handle, cubin);
+  return __nvJitLinkGetLinkedCubin_13_0 (handle, cubin);
 }
 #endif
  
-extern nvJitLinkResult __nvJitLinkGetLinkedPtxSize_12_9(
+extern nvJitLinkResult __nvJitLinkGetLinkedPtxSize_13_0(
   nvJitLinkHandle handle,
   size_t *size);
 /**
@@ -374,11 +366,11 @@ static inline nvJitLinkResult nvJitLinkGetLinkedPtxSize(
   nvJitLinkHandle handle,
   size_t *size)
 {
-  return __nvJitLinkGetLinkedPtxSize_12_9 (handle, size);
+  return __nvJitLinkGetLinkedPtxSize_13_0 (handle, size);
 }
 #endif
 
-extern nvJitLinkResult __nvJitLinkGetLinkedPtx_12_9(
+extern nvJitLinkResult __nvJitLinkGetLinkedPtx_13_0(
   nvJitLinkHandle handle,
   char *ptx);
 /**
@@ -401,11 +393,11 @@ static inline nvJitLinkResult nvJitLinkGetLinkedPtx(
   nvJitLinkHandle handle,
   char *ptx)
 {
-  return __nvJitLinkGetLinkedPtx_12_9 (handle, ptx);
+  return __nvJitLinkGetLinkedPtx_13_0 (handle, ptx);
 }
 #endif
  
-extern nvJitLinkResult __nvJitLinkGetErrorLogSize_12_9(
+extern nvJitLinkResult __nvJitLinkGetErrorLogSize_13_0(
   nvJitLinkHandle handle,
   size_t *size);
 /**
@@ -426,11 +418,11 @@ static inline nvJitLinkResult nvJitLinkGetErrorLogSize(
   nvJitLinkHandle handle,
   size_t *size)
 {
-  return __nvJitLinkGetErrorLogSize_12_9 (handle, size);
+  return __nvJitLinkGetErrorLogSize_13_0 (handle, size);
 }
 #endif
 
-extern nvJitLinkResult __nvJitLinkGetErrorLog_12_9(
+extern nvJitLinkResult __nvJitLinkGetErrorLog_13_0(
   nvJitLinkHandle handle,
   char *log);
 /**
@@ -452,11 +444,11 @@ static inline nvJitLinkResult nvJitLinkGetErrorLog(
   nvJitLinkHandle handle,
   char *log)
 {
-  return __nvJitLinkGetErrorLog_12_9 (handle, log);
+  return __nvJitLinkGetErrorLog_13_0 (handle, log);
 }
 #endif
 
-extern nvJitLinkResult __nvJitLinkGetInfoLogSize_12_9(
+extern nvJitLinkResult __nvJitLinkGetInfoLogSize_13_0(
   nvJitLinkHandle handle,
   size_t *size);
 /**
@@ -477,11 +469,11 @@ static inline nvJitLinkResult nvJitLinkGetInfoLogSize(
   nvJitLinkHandle handle,
   size_t *size)
 {
-  return __nvJitLinkGetInfoLogSize_12_9 (handle, size);
+  return __nvJitLinkGetInfoLogSize_13_0 (handle, size);
 }
 #endif
 
-extern nvJitLinkResult __nvJitLinkGetInfoLog_12_9(
+extern nvJitLinkResult __nvJitLinkGetInfoLog_13_0(
   nvJitLinkHandle handle,
   char *log);
 /**
@@ -503,7 +495,7 @@ static inline nvJitLinkResult nvJitLinkGetInfoLog(
   nvJitLinkHandle handle,
   char *log)
 {
-  return __nvJitLinkGetInfoLog_12_9 (handle, log);
+  return __nvJitLinkGetInfoLog_13_0 (handle, log);
 }
 #endif
 
