@@ -1,7 +1,7 @@
 //
 // NVIDIA_COPYRIGHT_BEGIN
 //
-// Copyright (c) 2014-2024, NVIDIA CORPORATION.  All rights reserved.
+// Copyright (c) 2014-2025, NVIDIA CORPORATION.  All rights reserved.
 //
 // NVIDIA CORPORATION and its licensors retain all intellectual property
 // and proprietary rights in and to this software, related documentation
@@ -55,7 +55,8 @@ typedef enum {
   NVRTC_ERROR_NO_PCH_CREATE_ATTEMPTED = 13,
   NVRTC_ERROR_PCH_CREATE_HEAP_EXHAUSTED = 14,
   NVRTC_ERROR_PCH_CREATE = 15,
-  NVRTC_ERROR_CANCELLED = 16
+  NVRTC_ERROR_CANCELLED = 16,
+  NVRTC_ERROR_TIME_TRACE_FILE_WRITE_FAILED = 17
 } nvrtcResult;
 
 
@@ -303,37 +304,6 @@ nvrtcResult nvrtcGetCUBINSize(nvrtcProgram prog, size_t *cubinSizeRet);
  */
 nvrtcResult nvrtcGetCUBIN(nvrtcProgram prog, char *cubin);
 
-
-#if defined(_WIN32)
-# define __DEPRECATED__(msg) __declspec(deprecated(msg))
-#elif (defined(__GNUC__) && (__GNUC__ < 4 || (__GNUC__ == 4 && __GNUC_MINOR__ < 5 && !defined(__clang__))))
-# define __DEPRECATED__(msg) __attribute__((deprecated))
-#elif (defined(__GNUC__))
-# define __DEPRECATED__(msg) __attribute__((deprecated(msg)))
-#else
-# define __DEPRECATED__(msg)
-#endif
-
-/**
- * \ingroup compilation
- * \brief   
- * DEPRECATION NOTICE: This function will be removed in a future release. Please use
- * nvrtcGetLTOIRSize (and nvrtcGetLTOIR) instead.
- */
-__DEPRECATED__("This function will be removed in a future release. Please use nvrtcGetLTOIRSize instead")
-nvrtcResult nvrtcGetNVVMSize(nvrtcProgram prog, size_t *nvvmSizeRet);
-
-/**
- * \ingroup compilation
- * \brief   
- * DEPRECATION NOTICE: This function will be removed in a future release. Please use
- * nvrtcGetLTOIR (and nvrtcGetLTOIRSize) instead.
- */
-__DEPRECATED__("This function will be removed in a future release. Please use nvrtcGetLTOIR instead")
-nvrtcResult nvrtcGetNVVM(nvrtcProgram prog, char *nvvm);
-
-#undef __DEPRECATED__
-
 /**
  * \ingroup compilation
  * \brief   nvrtcGetLTOIRSize sets the value of \p LTOIRSizeRet with the size of the LTO IR
@@ -629,10 +599,10 @@ nvrtcResult nvrtcSetFlowCallback(nvrtcProgram prog, int (*callback)(void*, void*
  * Short option names can be used instead of long option names.
  * When a compile option takes an argument, an assignment operator (\c =)
  * is used to separate the compile option argument from the compile option
- * name, e.g., \c "--gpu-architecture=compute_60".
+ * name, e.g., \c "--gpu-architecture=compute_100".
  * Alternatively, the compile option name and the argument can be specified in
  * separate strings without an assignment operator, .e.g,
- * \c "--gpu-architecture" \c "compute_60".
+ * \c "--gpu-architecture" \c "compute_100".
  * Single-character short option names, such as \c -D, \c -U, and \c -I, do
  * not require an assignment operator, and the compile option name and the
  * argument can be present in the same string with or without spaces between
@@ -648,14 +618,6 @@ nvrtcResult nvrtcSetFlowCallback(nvrtcProgram prog, int (*callback)(void*, void*
  *       Specify the name of the class of GPU architectures for which the
  *       input must be compiled.\n
  *       - Valid <c>\<arch\></c>s:
- *         - \c compute_50
- *         - \c compute_52
- *         - \c compute_53
- *         - \c compute_60
- *         - \c compute_61
- *         - \c compute_62
- *         - \c compute_70
- *         - \c compute_72
  *         - \c compute_75
  *         - \c compute_80
  *         - \c compute_86
@@ -665,7 +627,7 @@ nvrtcResult nvrtcSetFlowCallback(nvrtcProgram prog, int (*callback)(void*, void*
  *         - \c compute_90a
  *         - \c compute_100
  *         - \c compute_100f
- *         - \c compute_100a
+ *         - \c compute_100a 
  *         - \c compute_101
  *         - \c compute_101f
  *         - \c compute_101a
@@ -678,14 +640,6 @@ nvrtcResult nvrtcSetFlowCallback(nvrtcProgram prog, int (*callback)(void*, void*
  *         - \c compute_121
  *         - \c compute_121f
  *         - \c compute_121a
- *         - \c sm_50
- *         - \c sm_52
- *         - \c sm_53
- *         - \c sm_60
- *         - \c sm_61
- *         - \c sm_62
- *         - \c sm_70
- *         - \c sm_72
  *         - \c sm_75
  *         - \c sm_80
  *         - \c sm_86
@@ -708,7 +662,7 @@ nvrtcResult nvrtcSetFlowCallback(nvrtcProgram prog, int (*callback)(void*, void*
  *         - \c sm_121
  *         - \c sm_121f
  *         - \c sm_121a
- *       - Default: \c compute_52
+ *       - Default: \c compute_75
  *   - Separate compilation / whole-program compilation
  *     - \c --device-c (\c -dc)
  *
@@ -861,7 +815,7 @@ nvrtcResult nvrtcSetFlowCallback(nvrtcProgram prog, int (*callback)(void*, void*
  *      
  *      The user specified random seed will be used to replace random numbers used 
  *      in generating symbol names and variable names. The option can be used to 
- *      generate deterministicly identical ptx and object files. If the input value
+ *      generate deterministically identical ptx and object files. If the input value
  *      is a valid number (decimal, octal, or hex), it will be used directly as the
  *      random seed. Otherwise, the CRC value of the passed string will be used instead.
  * 
