@@ -625,7 +625,7 @@ static int nvfs_mgroup_mmap_internal(struct file *filp, struct vm_area_struct *v
         nvfs_mgroup_ptr_t nvfs_mgroup, nvfs_new_mgroup;
 	struct nvfs_gpu_args *gpu_info;
 	int os_pages_count;
-	vm_flags_t vm_flags;
+	vm_flags_t vm_flags, vm_flags_to_set;
 
 	nvfs_stat64(&nvfs_n_mmap);
         /* check length - do not allow larger mappings than the number of
@@ -668,11 +668,13 @@ static int nvfs_mgroup_mmap_internal(struct file *filp, struct vm_area_struct *v
                goto error;
         }
 
+ 
+	vm_flags_to_set = VM_MIXEDMAP | VM_DONTEXPAND | VM_DONTDUMP | VM_DONTCOPY;
         /* dont allow mremap to expand and dont allow copy on fork */
 #ifdef NVFS_VM_FLAGS_NOT_CONSTANT
-        vma->vm_flags |= VM_IO | VM_MIXEDMAP | VM_DONTEXPAND | VM_DONTDUMP | VM_DONTCOPY;
+        vma->vm_flags |= vm_flags_to_set;
 #else
-        vm_flags_set(vma, VM_IO | VM_MIXEDMAP | VM_DONTEXPAND | VM_DONTDUMP | VM_DONTCOPY);
+        vm_flags_set(vma, vm_flags_to_set);
 #endif
         vma->vm_ops = &nvfs_mmap_ops;
         nvfs_new_mgroup = (nvfs_mgroup_ptr_t)kzalloc(sizeof(struct nvfs_io_mgroup), GFP_KERNEL);
